package com.napa.pulse.service.impl;


import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.napa.pulse.dto.FiledLineGroupCode;
import com.napa.pulse.dto.OrderCycleData;
import com.napa.pulse.entity.pulseui.MinMaxClacLn;
import com.napa.pulse.entity.pulseui.MinMaxSitePref;
import com.napa.pulse.entity.pulseui.Site;
import com.napa.pulse.repository.MinMaxSitePrefRepository;
import com.napa.pulse.repository.OrderCycleRepository;
import com.napa.pulse.repository.SiteRepository;
import com.napa.pulse.security.GCPAuthenticationService;
import com.napa.pulse.service.interfaces.OrderCycleService;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.configurationprocessor.json.JSONObject;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.net.ssl.SSLContext;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class OrderCycleServiceImpl implements OrderCycleService {

    private static final Logger LOGGER = LoggerFactory.getLogger(OrderCycleServiceImpl.class);

    @Value("${orderCycle.service.url}")
    private String orderCycleTestUrl;

    //need to create
    private GCPAuthenticationService gcpAuthenticationService;

    @Autowired
    private SiteRepository siteRepo;

    @Autowired
    private OrderCycleRepository orderCycleRepo;


    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private MinMaxSitePrefRepository mmspRepo;

    @Autowired
    private OrderCycleServiceImpl(GCPAuthenticationService gcpAuthenticationService) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        this.gcpAuthenticationService = gcpAuthenticationService;
    }
    TrustStrategy acceptingTrustStrategy = (x509Certificates, s) -> true;
    SSLContext sslContext = org.apache.http.ssl.SSLContexts.custom().loadTrustMaterial(null, acceptingTrustStrategy).build();
    CloseableHttpClient httpClient = HttpClients.custom().setSSLContext(sslContext).build();
    HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory(httpClient);

    RestTemplate restTemplate = new RestTemplate(requestFactory);

    public OrderCycleServiceImpl() throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
    }

    @Override
//    OrderCycleData
    public OrderCycleData validateOnPremCloudConnections(List<String> sites) {
        try {
            HttpHeaders httpHeaders = new HttpHeaders();
            httpHeaders.add("Authorization", "Bearer ".concat(gcpAuthenticationService.getToken()));
            HttpEntity<Object> httpEntity = new HttpEntity<>(httpHeaders);
            JSONObject jsonObject = new JSONObject(restTemplate.exchange(orderCycleTestUrl.concat("?sites=" + String.join(",", sites)),
                    HttpMethod.GET, httpEntity, String.class).getBody())
                    .getJSONObject("content");

            OrderCycleData orderCycleResponse = OrderCycleData.builder().
                    orderCycleData(new Gson().fromJson(String.valueOf(jsonObject), new TypeToken<Map<String, List<FiledLineGroupCode>>>(){}.getType())).build();


            saveOrderCycle(orderCycleResponse,sites);

            return orderCycleResponse;
        } catch (Exception ex) {
            LOGGER.error("Exception in OrderCycleService Call: ", ex);
            return null;
        }

    }

    private void saveOrderCycle(OrderCycleData orderCycleData, List<String>sites) {
//        List<Integer> site =sites.stream().map(Integer::parseInt).collect(Collectors.toList());

        List<Site> siteData = siteRepo.findAllByIdNumberIn(sites);
        String siteList = siteData.stream().map(siteInfo -> String.valueOf(siteInfo.getSiteId())).collect(Collectors.joining(","));
        String sql = "update MIN_MAX_SITE_PREFERENCES set ORDER_CYCLE_STATUS='N' where site_id in ( " + siteList + " ) ";
        jdbcTemplate.update(sql);
        orderCycleData.getOrderCycleData().forEach((key, value) -> {
            {
                int siteId = siteData.stream().filter(siteInfo -> siteInfo.getIdNumber().equalsIgnoreCase(key)).findFirst().map(Site::getSiteId).orElse(0);

                for (FiledLineGroupCode FiledLineGroupCode : value) {

                    MinMaxClacLn obj = orderCycleRepo.getMinMaxClacLnByFieldLine(FiledLineGroupCode.getFldLineAbbr());

//                    if (obj != null) {
//                        MinMaxSitePref objPref = new MinMaxSitePref();
//                        objPref.setSiteId(siteId);
//                        objPref.setLineId(obj.getLineId());
//                        int orderCycleValue = FiledLineGroupCode.getDcOrderCycle() != null ? FiledLineGroupCode.getDcOrderCycle() : FiledLineGroupCode.getVndrOrderCycle() != null ? FiledLineGroupCode.getVndrOrderCycle() : 7;
//                        objPref.setOrderCycle(orderCycleValue);
//                        objPref.setOrderCycleStatus('Y');
//
//                        mmspRepo.save(objPref);
//                    }
                    if (obj != null) {
                        MinMaxSitePref objPref = new MinMaxSitePref();
                        objPref.setSiteId(siteId);
                        objPref.setLineId(obj.getLineId());
                        Optional<MinMaxSitePref> test=mmspRepo.findByCompositeKey(objPref.getSiteId(),objPref.getLineId());
                        if(test.isPresent())
                        {
                            MinMaxSitePref det = test.get();
                            int orderCycleValue = FiledLineGroupCode.getDcOrderCycle() != null ? FiledLineGroupCode.getDcOrderCycle() : FiledLineGroupCode.getVndrOrderCycle() != null ? FiledLineGroupCode.getVndrOrderCycle() : 7;
                            det.setOrderCycle(orderCycleValue);
                            det.setOrderCycleStatus('Y');
                            mmspRepo.save(det);
                        }
                        else {
                            MinMaxSitePref objPref1 = new MinMaxSitePref();
                            objPref1.setSiteId(siteId);
                            objPref1.setLineId(obj.getLineId());

                            int orderCycleValue = FiledLineGroupCode.getDcOrderCycle() != null ? FiledLineGroupCode.getDcOrderCycle() : FiledLineGroupCode.getVndrOrderCycle() != null ? FiledLineGroupCode.getVndrOrderCycle() : 7;

                            objPref1.setOrderCycle(orderCycleValue);
                            objPref1.setOrderCycleStatus('Y');
                            mmspRepo.save(objPref1);


                        }



                    }
                }
            }
        });

    }

}
